//          File: graph.cpp
//        Author: Fred Sisenda and Samuel Sandeen
// Last Modified: March 24, 2012

#include<cstdlib>
#include "graph.h"

using namespace std;

// Constructor
// Creates empty undirected graph
Graph::Graph(){
	numV = 0;
	numE = 0;
}


// Constructor
// Creates complete undirected graph with n vertices
// Vertex ids from 0 to n-1
// weights are random from 1-1000
// rand() not randomized so you get the same graph on subsequent runs
// Needs to be fast for big graphs so terrible things are done to save time
Graph::Graph(unsigned int n){
	// setup vertices
	numV = n;
	vertices = vector<Vertex*>(n);
	pseudo_map = vector<Vertex*>(n);
	// pre-allocate vertices
	
	V.resize(numV);
	
	Vertex* v;
	for (unsigned int ii=0; ii<n; ii++){
		v = &V[ii];
		// setup vertex instance variables
		v->id = ii;
		v->index = ii;
		v->numE = n-1;
		v->adjV = vector<Vertex*>(n-1);
		v->adjW = vector<int>(n-1);
		v->adjE = vector<Edge*>(n-1);
		// add vertex to vertex lists
		vertices[ii] = v;
		pseudo_map[ii] = v;
	}
	
	// setup edges
	numE = n*(n-1)/2;
	edges = vector<Edge*>(numE);
	//pre-allocate edges
	E.resize(numE);
	
	int w, kk = 0;
	Edge* e;
	Vertex* u;
	for (unsigned int ii = 0; ii<n-1; ii++){
		u = getVertexByIndex(ii);
		for (unsigned int jj = ii+1; jj<n; jj++){
			e = &E[kk];
			v = getVertexByIndex(jj);
			w = rand() % 1000 + 1;
			// setup edge instance  variables
			e->index = kk;
			e->weight = w;
			e->u = u;
			e->v = v;
			// connect vertices
			u->adjV[jj-1] = v;
			u->adjW[jj-1] = w;
			u->adjE[jj-1] = e;
			v->adjV[ii] = u;
			v->adjW[ii] = w;
			v->adjE[ii] = e;
			// add edge to edge list
			edges[kk] = e; 
			kk++;
		}
	}
}


// Returns the pointer to vertex with specified id
// Returns NULL if no vertex with id exists in graph
Vertex* Graph::getVertexById(unsigned int id){
	if (id>=getNumVertices()) pseudo_map.resize(id+1,NULL);
	return pseudo_map[id];
}


// Returns the pointer to nth vertex in graph 
// Returns NULL if n exceeds range
Vertex* Graph::getVertexByIndex(unsigned int n){
	if (n>=getNumVertices()) return NULL;
	return vertices[n];
}


// Returns the pointer to nth edge in graph 
// Returns NULL if n exceeds range
Edge* Graph::getEdgeByIndex(unsigned int n){
	if (n>=getNumEdges()) return NULL;
	return edges[n];
}


// Return current number of vertices in graph
unsigned int Graph::getNumVertices(){
	return numV;
}


// Return current number of edges in graph
unsigned int Graph::getNumEdges(){
	return numE;
}


// Add vertex with specified id to graph
// If vertex with id already exists, does nothing, returns false'
// Otherwise the vertex is added, returns 'true'
bool Graph::addVertex(int id){ 
	// check if the vertex is already in the graph
	if ( getVertexById(id) != NULL ) return false;
	
	// allocate memory for new vertex
	unsigned int n = getNumVertices();
	V.push_back(Vertex(id,n));
	Vertex* newVertex = &(V.back());
	
	// add vertex to vertex list
	numV++;
	vertices.push_back(newVertex);
	pseudo_map[id] = newVertex;
	return true;
}


// Adds undirected edge between v1 and v2 of weight w
// v1 and v2 have to exist in graph
// Returns true if successful'
// Returns false if v1 or v2 does not exist in graph
bool Graph::addEdge(int v1Id, int v2Id, int w) {
	// find v1
	Vertex* v1 = getVertexById(v1Id);
	if (v1 == NULL) return false;
	
	// find v2
	Vertex* v2 = getVertexById(v2Id);
	if (v2 == NULL) return false;
	
	// allocate memory for new edge
	int n = getNumEdges();
	E.push_back(Edge(n,v1,v2,w));
	Edge* newEdge = &(E.back());
	edges.push_back(newEdge);
	
	// add edge
	v1->addEdgeTo(v2,w,newEdge);
	v2->addEdgeTo(v1,w,newEdge);
	numE++;
	return true;
}


// Prints all the edges/vertices adjacent to vertex v
void Graph::printEdges(Vertex* v){
	int n = v->getNumEdges();
	int id = v->getId();
	if (n==0)
		cout << "vertex " << id << " has no edges" << endl << endl;
	else{
		cout << "vertex " << id << " has edges to: " << endl;
		for(int ii=0; ii<n; ii++){
			int targetId = (v->getEdgeDest(ii))->getId();
			int weight = v->getEdgeWeight(ii);
			cout << "\t" << targetId << " of weight " << weight << endl;
		}
		cout << endl;
	}
}


// Print edge e
void Graph::printEdge(Edge* e){
	int uId = (e->getEnd1())->getId();
	int vId = (e->getEnd2())->getId();
	cout << "weight: " << e->getWeight() << "\tedge: (" << uId << "," << vId << ")" << endl;
}


// Print adjacency of all vertices in the graph
void Graph::print(){
	vector<Vertex*>::const_iterator lIter;
	for(lIter=vertices.begin(); lIter!=vertices.end(); lIter++)
		printEdges(*lIter);
}

const std::vector<Edge*>& Graph::getEdges() const{
	return edges;
}